import json
from typing import Type, TypeVar, Dict, Any, List

from ..models.request_message_topic import RequestMessageTopic
from ..models.twitter_user_metadata import TwitterUserMetadata
from ...domain.models.hai_message import HaiMessage

T = TypeVar('T', bound='HaiMessage')

class TwitterGetUsersResponseMessage(HaiMessage):

    @classmethod
    def create(cls: Type[T], topic: str, payload: Dict[Any, Any]) -> T:
        """Create a message - inherited from HaiMessage"""
        return super().create(topic=topic, payload=payload)

    @classmethod
    def create_message(cls, users: List[dict[str, str]]) -> 'TwitterGetUsersResponseMessage':
        user_result = json.dumps(users)
        return cls.create(
            topic=RequestMessageTopic.TWITTER_GET_USERS_RESPONSE,
            payload={
                "users": user_result,
            },
        )

    @property
    def users(self) -> List[TwitterUserMetadata]:
        """Get the collection name from the message payload"""
        users = self.payload.get("users", "")
        users_dict = json.loads(users)
        response_list = []
        for user in users_dict:
            response_list.append(TwitterUserMetadata.from_dict(user))
        return response_list

    @classmethod
    def from_hai_message(cls, message: HaiMessage) -> 'TwitterGetUsersResponseMessage':
        payload = message.payload
        metadata = payload.get("users", "")
        meta_list = json.loads(metadata)
        return cls.create_message(
            users=meta_list
        )


from ...domain.models.twitter_user_metadata import TwitterUserMetadata


class TwitterMessageMetaData:
    def __init__(self, tweet_id: str, user: TwitterUserMetadata, message:  str, created_at: str, view_count: int, retweet_count: int, reply_count: int):
        self.tweet_id = tweet_id
        self.message = message
        self.user = user
        self.created_at = created_at,
        self.view_count = view_count
        self.retweet_count = retweet_count
        self.reply_count = reply_count

    def to_dict(self) -> dict:
        """
        Convert TwitterMessage instance to a dictionary.

        Returns:
            dict: A dictionary representation of the TwitterMessage instance.
            If the user attribute is a TwitterUser instance, it will be converted
            to a dictionary using its __dict__ attribute.
        """
        return {
            'tweet_id': self.tweet_id,
            'message': self.message,
            'created_at': self.created_at,
            'view_count': self.view_count,
            'retweet_count': self.retweet_count,
            'reply_count': self.reply_count,
            'user': self.user.to_dict()
        }


from typing import Type, TypeVar, Dict, Any, Optional

from ..models.request_message_topic import RequestMessageTopic
from ...domain.models.hai_message import HaiMessage

T = TypeVar('T', bound='HaiMessage')

class WebGetDocsRequestMessage(HaiMessage):

    @classmethod
    def create(cls: Type[T], topic: str, payload: Dict[Any, Any]) -> T:
        """Create a message - inherited from HaiMessage"""
        return super().create(topic=topic, payload=payload)

    @classmethod
    def create_message(cls, collection_name: str, twitter_user: Optional[str], root_url: Optional[str]) -> 'WebGetDocsRequestMessage':
        return cls.create(
            topic=RequestMessageTopic.WEB_GET_DOCS,
            payload={
                "twitter_user": twitter_user,
                "root_url": root_url,
                "collection_name": collection_name,
            },
        )

    @property
    def twitter_user(self) -> str:
        return self.payload.get("twitter_user", "")

    @property
    def root_url(self) -> str:
        return self.payload.get("root_url", "")

    @property
    def collection_name(self) -> str:
        return self.payload.get("collection_name", "")

    @classmethod
    def from_hai_message(cls, message: HaiMessage) -> 'WebGetDocsRequestMessage':
        payload = message.payload

        return cls.create_message(
            twitter_user=payload.get("twitter_user", ""),
            root_url=payload.get("root_url", ""),
            collection_name=payload.get("collection_name", "")
        )


class TwitterUserMetadata:
    def __init__(self,
                 id: str,
                 screen_name: str,
                 user_name: str = None,
                 description: str = None,
                 followers_count: int = 0,
                 like_count: int = 0,
                 is_verified: bool = False,
                 url: str = None,
                 bio_urls: list = None):
        """
        Initialize a TwitterUserMetadata instance.

        Args:
            id (str): The user's Twitter ID
            screen_name (str): The user's screen name
            user_name (str, optional): The user's name
            description (str, optional): The user's bio description
            followers_count (int, optional): Number of followers
            like_count (int, optional): Number of likes
            is_verified (bool, optional): Whether the user is verified
            url (str, optional): The user's profile URL
            bio_urls (list, optional): URLs mentioned in the user's bio
        """
        self.id = id
        self.screen_name = screen_name
        self.user_name = user_name
        self.description = description
        self.followers_count = followers_count
        self.like_count = like_count
        self.is_verified = is_verified
        self.url = url
        self.bio_urls = bio_urls or []

    def to_dict(self) -> dict:
        """
        Convert TwitterUserMetadata instance to a dictionary.

        Returns:
            dict: A dictionary representation of the TwitterUserMetadata instance.
        """
        return {
            'id': self.id,
            'screen_name': self.screen_name,
            'user_name': self.user_name,
            'description': self.description,
            'followers_count': self.followers_count,
            'like_count': self.like_count,
            'is_verified': self.is_verified,
            'url': self.url,
            'bio_urls': self.bio_urls
        }

    @classmethod
    def from_dict(cls, data: dict) -> 'TwitterUserMetadata':
        """
        Create a TwitterUserMetadata instance from a dictionary.

        Args:
            data (dict): A dictionary containing TwitterUserMetadata attributes

        Returns:
            TwitterUserMetadata: A new TwitterUserMetadata instance
        """
        return cls(
            id=data.get('id'),
            screen_name=data.get('screen_name'),
            user_name=data.get('user_name'),
            description=data.get('description'),
            followers_count=data.get('followers_count', 0),
            like_count=data.get('like_count', 0),
            is_verified=data.get('is_verified', False),
            url=data.get('url'),
            bio_urls=data.get('bio_urls', [])
        )


import json
from typing import Type, TypeVar, Dict, Any

from ..models.request_message_topic import RequestMessageTopic
from ..models.vector_query_answer import VectorQueryAnswer
from ...domain.models.hai_message import HaiMessage

T = TypeVar('T', bound='HaiMessage')

class VectorQueryCollectionResponseMessage(HaiMessage):
    """Response Message from reading vector data"""

    @classmethod
    def create(cls: Type[T], topic: str, payload: Dict[Any, Any]) -> T:
        """Create a message - inherited from HaiMessage"""
        return super().create(topic=topic, payload=payload)

    @classmethod
    def create_message(cls, result: dict[int, dict[str, str]]) -> 'VectorQueryCollectionResponseMessage':
        """Create a message requesting Twitter user data"""
        result_list = json.dumps(result)
        return cls.create(
            topic=RequestMessageTopic.VECTORS_QUERY_RESPONSE,
            payload={
                "result": result_list,
            },
        )

    @property
    def results(self) -> list[VectorQueryAnswer]:
        """Returns the results from the message payload"""
        results = self.payload.get("result", "")
        meta_dict = json.loads(results)
        response_list = []
        for meta in meta_dict:
            response_list.append(VectorQueryAnswer.from_dict(meta))
        return response_list

    @classmethod
    def from_hai_message(cls, message: HaiMessage) -> 'VectorQueryCollectionResponseMessage':
        # Extract necessary fields from the message payload
        payload = message.payload
        result = payload.get("result", "")
        result_list = json.loads(result)
        return cls.create_message(
           result=result_list
        )



from typing import TypeVar, Dict, Any, Type, List

from ..models.request_message_topic import RequestMessageTopic
from ...domain.models.hai_message import HaiMessage

T = TypeVar('T', bound='HaiMessage')

class GraphGetAllResultResponseMessage(HaiMessage):
    """Message containing all nodes and relationships from the graph"""

    @classmethod
    def create(cls: Type[T], topic: str, payload: Dict[Any, Any]) -> T:
        """Create a message - inherited from HaiMessage"""
        return super().create(topic=topic, payload=payload)

    @classmethod
    def create_message(cls, nodes: List[Dict], relationships: List[Dict]) -> 'GraphGetAllResultResponseMessage':
        """Create a message with all nodes and relationships"""
        return cls.create(
            topic=RequestMessageTopic.GRAPH_GET_ALL_RESPONSE,
            payload={
                "nodes": nodes,
                "relationships": relationships
            },
        )

    @property
    def nodes(self) -> List[Dict]:
        """Get the list of nodes from the payload"""
        return self.payload.get("nodes", [])

    @property
    def relationships(self) -> List[Dict]:
        """Get the list of relationships from the payload"""
        return self.payload.get("relationships", [])

    @classmethod
    def from_hai_message(cls, message: HaiMessage) -> 'GraphGetAllResultResponseMessage':
        # Extract the necessary fields from the message payload
        payload = message.payload

        return cls.create_message(
            nodes=payload.get("nodes", []),
            relationships=payload.get("relationships", [])
        )



from typing import TypeVar, Dict, Any, Type

from ..models.request_message_topic import RequestMessageTopic
from ...domain.models.hai_message import HaiMessage

T = TypeVar('T', bound='HaiMessage')

class GraphNodesByPropertyRequestMessage(HaiMessage):
    """Message to get nodes with a specific property value and their relationships"""

    @classmethod
    def create(cls: Type[T], topic: str, payload: Dict[Any, Any]) -> T:
        """Create a message - inherited from HaiMessage"""
        return super().create(topic=topic, payload=payload)

    @classmethod
    def create_message(cls, property_name: str, property_value: any) -> 'GraphNodesByPropertyRequestMessage':
        """Create a message requesting nodes with a specific property value"""
        return cls.create(
            topic=RequestMessageTopic.GRAPH_NODES_BY_PROPERTY,
            payload={
                "property_name": property_name,
                "property_value": property_value
            },
        )

    @property
    def property_name(self) -> str:
        """Get the property name from the payload"""
        return self.payload.get("property_name")

    @property
    def property_value(self) -> any:
        """Get the property value from the payload"""
        return self.payload.get("property_value")

    @classmethod
    def from_hai_message(cls, message: HaiMessage) -> 'GraphNodesByPropertyRequestMessage':
        # Extract the necessary fields from the message payload
        payload = message.payload

        return cls.create_message(
            property_name=payload.get("property_name", ''),
            property_value=payload.get("property_value")
        )


from abc import ABC, abstractmethod
from typing import Callable


class MessageSubscriber(ABC):
    """
    Abstract base class representing a message subscriber.

    This class defines the contract for subscribing to and unsubscribing from
    message topics. Subclasses are required to implement the methods for
    subscribing to topics with message handlers and unsubscribing from all
    active subscriptions.

    """

    @abstractmethod
    async def subscribe(self, topic: str, handler: Callable) -> None:
        """Subscribe to a topic with a message handler."""
        pass

    @abstractmethod
    async def unsubscribe_all(self) -> None:
        """Unsubscribe from all subscriptions."""
        pass




from typing import TypeVar, Dict, Any, Type, List

from ..models.request_message_topic import RequestMessageTopic
from ...domain.models.hai_message import HaiMessage

T = TypeVar('T', bound='HaiMessage')

class GraphGetAllRequestMessage(HaiMessage):
    """Message to get all nodes and relationships from the graph"""

    @classmethod
    def create(cls: Type[T], topic: str, payload: Dict[Any, Any]) -> T:
        """Create a message - inherited from HaiMessage"""
        return super().create(topic=topic, payload=payload)

    @classmethod
    def create_message(cls) -> 'GraphGetAllRequestMessage':
        """Create a message requesting to get all nodes and relationships"""
        return cls.create(
            topic=RequestMessageTopic.GRAPH_GET_ALL,
            payload={}
        )

    @classmethod
    def from_hai_message(cls, message: HaiMessage) -> 'GraphGetAllRequestMessage':
        return cls.create_message()



from typing import Type, TypeVar, Dict, Any

from ..models.request_message_topic import RequestMessageTopic
from ...domain.models.hai_message import HaiMessage

T = TypeVar('T', bound='HaiMessage')

class TwitterGetUserRequestMessage(HaiMessage):
    """Message to request Twitter user information"""

    @classmethod
    def create(cls: Type[T], topic: str, payload: Dict[Any, Any]) -> T:
        """Create a message - inherited from HaiMessage"""
        return super().create(topic=topic, payload=payload)

    @classmethod
    def create_message(cls, username: str) -> 'TwitterGetUserRequestMessage':
        """Create a message requesting Twitter user data"""
        return cls.create(
            topic=RequestMessageTopic.TWITTER_GET_USER,
            payload={"username": username},
        )

    @property
    def username(self) -> str:
        """Get the username from the payload"""
        return self.payload.get("username", "")

    @classmethod
    def from_hai_message(cls, message: HaiMessage) -> 'TwitterGetUserRequestMessage':
        payload = message.payload

        return cls.create_message(
        username=payload.get("username", "")
        )



import json
from typing import Type, TypeVar, Dict, Any, List

from ..models.request_message_topic import RequestMessageTopic
from ...domain.models.hai_message import HaiMessage

T = TypeVar('T', bound='HaiMessage')

class TwitterGetUsersRequestMessage(HaiMessage):
    """Message to request Twitter user information"""

    @classmethod
    def create(cls: Type[T], topic: str, payload: Dict[Any, Any]) -> T:
        """Create a message - inherited from HaiMessage"""
        return super().create(topic=topic, payload=payload)

    @classmethod
    def create_message(cls, usernames: List[str]) -> 'TwitterGetUsersRequestMessage':
        """Create a message requesting Twitter user data"""
        return cls.create(
            topic=RequestMessageTopic.TWITTER_GET_USERS,
            payload={"usernames": json.dumps(usernames)},
        )

    @property
    def usernames(self) -> List[str]:
        """Get the username from the payload"""
        usernames_payload = self.payload.get("usernames", "")
        usernames = json.loads(usernames_payload)
        response_list = []
        for username in usernames:
            response_list.append(username)
        return response_list

    @classmethod
    def from_hai_message(cls, message: HaiMessage) -> 'TwitterGetUsersRequestMessage':
        payload = message.payload
        usernames = payload.get("usernames", "")
        usernames_list = json.loads(usernames)
        return cls.create_message(
        usernames=usernames_list
        )


from dataclasses import dataclass
from typing import TypeVar, Dict, Any, Type, List

from ..models.request_message_topic import RequestMessageTopic
from ..models.twitter_message_metadata import TwitterMessageMetaData
from ...domain.models.hai_message import HaiMessage

T = TypeVar('T', bound='HaiMessage')
@dataclass
class TwitterUserTweetsResponseMessage(HaiMessage):
    """Message containing tweets from a Twitter user"""

    @classmethod
    def create(cls: Type[T], topic: str, payload: Dict[Any, Any]) -> T:
        """Create a message - inherited from HaiMessage"""
        return super().create(topic=topic, payload=payload)

    @classmethod
    def create_message(cls, user_id: str, tweets: List[Dict[str, Any]],
                       success: bool = True, error: str = None) -> 'TwitterUserTweetsResponseMessage':
        """Create a message with tweets from a Twitter user"""
        return cls.create(
            topic=RequestMessageTopic.TWITTER_GET_USER_TWEETS_RESPONSE,
            payload={
                "user_id": user_id,
                "tweets": tweets,
                "success": success,
                "error": error
            },
        )

    @property
    def user_id(self) -> str:
        """Get the user ID from the payload"""
        return self.payload.get("user_id")

    @property
    def tweets(self) -> List[Dict[str, Any]]:
        """Get the tweets from the payload"""
        return self.payload.get("tweets", [])

    @property
    def success(self) -> bool:
        """Get whether the request was successful"""
        return self.payload.get("success", False)

    @property
    def error(self) -> str:
        """Get any error message"""
        return self.payload.get("error")

    @classmethod
    def from_hai_message(cls, message: HaiMessage) -> 'TwitterUserTweetsResponseMessage':
        """Create a TwitterUserTweetsResponseMessage from a HaiMessage"""
        payload = message.payload

        return cls.create_message(
            user_id=payload.get("user_id", ""),
            tweets=payload.get("tweets", []),
            success=payload.get("success", False),
            error=payload.get("error")
        )

    @classmethod
    def from_twitter_messages(cls, user_id: str,
                              twitter_messages: List[TwitterMessageMetaData]) -> 'TwitterUserTweetsResponseMessage':
        """Create a response message from a list of TwitterMessage objects"""
        tweets = []

        for message in twitter_messages:
            user = message.user

            tweet_data = {
                "tweet_id": str(message.tweet_id),
                "message": message.message,
                "created_at": message.created_at.isoformat() if message.created_at else None,
                "view_count": message.view_count,
                "retweet_count": message.retweet_count,
                "reply_count": message.reply_count,
                "user": message.user.to_dict() if user else None,
            }

            tweets.append(tweet_data)

        return cls.create_message(
            user_id=user_id,
            tweets=tweets,
            success=True
        )
